//==================================================================================
// The information contained herein is the exclusive property of
// Generalplus Technology Co. And shall not be distributed, reproduced,
// or disclosed in whole in part without prior written permission.
//       (C) COPYRIGHT 2010   Generalplus TECHNOLOGY CO.                            
//                   ALL RIGHTS RESERVED
// The entire notice above must be reproduced on all authorized copies.
//==================================================================================
//==========================================================================
// Program Name: PWMIO.asm
// Applied body: GPL95100UA Series Body
// Programmer  : 
// Description : PWMIO driver code
// History version
// Rev #     Date       Who     Comments
// -----  -----------  ------   --------------------------------------------
// 1.0       					Oringinal Version
// -----  -----------  ------   --------------------------------------------


//==================================================================================
// Include file area
//==================================================================================
.include GPL951_Body.inc
.include PWMIO_Define.inc

//=============================================
//Constant Define Area
//=============================================


//=============================================
//RAM Define Area
//=============================================
PWMIO_RAM:	.section	.ram

//=============================================
//Program Area
//=============================================
PWMIO_DriverCode: .section	.code


//=============================================
//Public declare Area
//=============================================
.public _PWMIO_Initial
.public F_PWMIO_Initial
.public _PWMIO_Duty_Setting
.public F_PWMIO_Duty_Setting
.public _PWMIO_Freq_Setting
.public F_PWMIO_Freq_Setting
.public _PWMIO_Start
.public F_PWMIO_Start
.public _PWMIO_Stop
.public F_PWMIO_Stop


//************************************************************//
//function : F_PWMIO_Initial
//Description : Initial PWMIO
//Parameter : none
//Return : none
//Destory : r1
//************************************************************//
_PWMIO_Initial: 			.proc
	push bp to [sp]
	call F_PWMIO_Initial
	pop bp from [sp]
	retf
F_PWMIO_Initial:
	r1 = [P_IOB_Mux]				//Enable PWMIO Module	
	r1 |= 0x0008
	[P_IOB_Mux] = r1
.if PWMIO0
	r1 = 0x8000						//IOB0 will output a square wave with 1/2 duty 
	[P_TimerA_CCP_Reg] = r1		
	r1 = 0x0000
	[P_TimerA_Preload] = r1			//65536
	r1 = C_PWM_En + PWMIO0_Polarity_setting
	[P_TimerA_CCP_Ctrl] = r1
	r1 = C_TimerFlag  + C_TimerSrcB_High + C_TimerSrcA_SYSdiv2
	[P_TimerA_Ctrl] = r1			//cycle and frequency is SYS/2/65536		
.endif	
.if PWMIO1
	r1 = 0x8000						//IOB1 will output a square wave with 1/2 duty 
	[P_TimerB_CCP_Reg] = r1		
	r1 = 0x0000
	[P_TimerB_Preload] = r1			//65536
	r1 = C_PWM_En + PWMIO1_Polarity_setting
	[P_TimerB_CCP_Ctrl] = r1
	r1 = C_TimerFlag  + C_TimerSrcB_High + C_TimerSrcA_SYSdiv2
	[P_TimerB_Ctrl] = r1			//cycle and frequency is SYS/2/65536	
.endif	
.if PWMIO2
	r1 = 0x8000						//IOB2 will output a square wave with 1/2 duty 
	[P_TimerC_CCP_Reg] = r1		
	r1 = 0x0000
	[P_TimerC_Preload] = r1			//65536
	r1 = C_PWM_En + PWMIO2_Polarity_setting
	[P_TimerC_CCP_Ctrl] = r1
	r1 = C_TimerFlag  + C_TimerSrcB_High + C_TimerSrcA_SYSdiv2
	[P_TimerC_Ctrl] = r1			//cycle and frequency is SYS/2/65536		
.endif	
.if PWMIO3
	r1 = 0x8000						//IOB3 will output a square wave with 1/2 duty 
	[P_TimerD_CCP_Reg] = r1		
	r1 = 0x0000
	[P_TimerD_Preload] = r1			//65536
	r1 = C_PWM_En + PWMIO3_Polarity_setting
	[P_TimerD_CCP_Ctrl] = r1
	r1 = C_TimerFlag  + C_TimerSrcB_High + C_TimerSrcA_SYSdiv2
	[P_TimerD_Ctrl] = r1			//cycle and frequency is SYS/2/65536		
.endif	
	retf
	.endp


//************************************************************//
//function : F_PWMIO_Duty_Setting
//Description : Set PWMIO duty
//Parameter : r1 : PWMIO select(High Byte) 
//			  r2 : PWMIO duty setting
//Return : none
//Destory : r1,r2
//************************************************************//
_PWMIO_Duty_Setting:			.proc
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]					//PWMIO select 
	r2=[bp+4]					//PWMIO duty 
	call F_PWMIO_Duty_Setting
	pop bp from [sp]
	retf
F_PWMIO_Duty_Setting:
	r1 &= 0xff00
	test r1,0x0100
	jz	L_PWMIO1_Sel?
	[P_TimerA_CCP_Reg] = r2	
L_PWMIO1_Sel?:	
	test r1,0x0200
	jz	L_PWMIO2_Sel?
	[P_TimerB_CCP_Reg] = r2
L_PWMIO2_Sel?:	
	test r1,0x0400
	jz	L_PWMIO3_Sel?
	[P_TimerC_CCP_Reg] = r2		
L_PWMIO3_Sel?:	
	test r1,0x0800
	jz	L_None_Sel?	
	[P_TimerD_CCP_Reg] = r2
L_None_Sel?:	
	retf			
	.endp	
	

//************************************************************//
//function : F_PWMIO_Freq_Setting
//Description : Set PWMIO Frequency
//Parameter : r1 : PWMIO select(High Byte) 
//			  r2 : PWMIO Frequency setting
//Return : none
//Destory : r1,r2
//************************************************************//
_PWMIO_Freq_Setting:			.proc
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]					//PWMIO select 
	r2=[bp+4]					//PWMIO Frequency 
	call F_PWMIO_Freq_Setting
	pop bp from [sp]
	retf
F_PWMIO_Freq_Setting:
	r1 &= 0xff00
	test r1,0x0100
	jz	L_PWMIO1_Sel?
	[P_TimerA_Preload] = r2	
L_PWMIO1_Sel?:	
	test r1,0x0200
	jz	L_PWMIO2_Sel?
	[P_TimerB_Preload] = r2
L_PWMIO2_Sel?:	
	test r1,0x0400
	jz	L_PWMIO3_Sel?
	[P_TimerC_Preload] = r2		
L_PWMIO3_Sel?:	
	test r1,0x0800
	jz	L_None_Sel?	
	[P_TimerD_Preload] = r2
L_None_Sel?:	
	retf			
	.endp	
		
	
//************************************************************//
//function : F_PWMIO_Start
//Description : Start PWMIO transmission
//Parameter : r1 : PWMIO select(High Byte)
//Return : none
//Destory : r1, r2
//************************************************************//
_PWMIO_Start:			.proc
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]					//PWMIO select
	call F_PWMIO_Start
	pop bp from [sp]
	retf
F_PWMIO_Start:
	r1 &= 0xff00
	test r1,0x0100
	jz	L_PWMIO1_Sel?
	r2 = [P_TimerA_Ctrl]
	r2 |= C_TimerEn
	[P_TimerA_Ctrl] = r2
L_PWMIO1_Sel?:	
	test r1,0x0200
	jz	L_PWMIO2_Sel?
	r2 = [P_TimerB_Ctrl]
	r2 |= C_TimerEn
	[P_TimerB_Ctrl] = r2
L_PWMIO2_Sel?:	
	test r1,0x0400
	jz	L_PWMIO3_Sel?
	r2 = [P_TimerC_Ctrl]
	r2 |= C_TimerEn
	[P_TimerC_Ctrl] = r2		
L_PWMIO3_Sel?:	
	test r1,0x0800
	jz	L_None_Sel?	
	r2 = [P_TimerD_Ctrl]
	r2 |= C_TimerEn
	[P_TimerD_Ctrl] = r2
L_None_Sel?:	
	retf			
	.endp
	
	
//************************************************************//
//function : F_PWMIO_Stop
//Description : End PWMIO transmission
//Parameter : r1 : PWMIO select(High Byte)
//Return : none
//Destory : r1, r2
//************************************************************//
_PWMIO_Stop:			.proc
	push	bp to [sp]
	bp=sp+1
	r1=[bp+3]					//PWMIO select
	call F_PWMIO_Stop
	pop bp from [sp]
	retf
F_PWMIO_Stop:
	r1 &= 0xff00
	test r1,0x0100
	jz	L_PWMIO1_Sel?
	r2 = [P_TimerA_Ctrl]
	r2 &= ~C_TimerEn
	[P_TimerA_Ctrl] = r2
L_PWMIO1_Sel?:	
	test r1,0x0200
	jz	L_PWMIO2_Sel?
	r2 = [P_TimerB_Ctrl]
	r2 &= ~C_TimerEn
	[P_TimerB_Ctrl] = r2
L_PWMIO2_Sel?:	
	test r1,0x0400
	jz	L_PWMIO3_Sel?
	r2 = [P_TimerC_Ctrl]
	r2 &= ~C_TimerEn
	[P_TimerC_Ctrl] = r2		
L_PWMIO3_Sel?:	
	test r1,0x0800
	jz	L_None_Sel?	
	r2 = [P_TimerD_Ctrl]
	r2 &= ~C_TimerEn
	[P_TimerD_Ctrl] = r2
L_None_Sel?:	
	retf			
	.endp	
	

	